;**********************************************************************
;                                 PIC12F1501
;                           -----------U-----------  
;                     +5V --|Vdd               Vss|-- GND
; ONSW (P-MOS gate drive) --|RA5/oscin     RA0/PGD|-- LEDR out - vstup erven LED
;       CHRG (overcharge) --|RA4/oscout    RA1/PGC|-- LEDG out - vstup zelen LED
; SETSW - tla. nastaven --|RA3/MCLR/Vpp  RA2/INT|-- SWITCH - vstup tlatko ON/OFF
;                           -----------------------
; fosc = 31 kHz, 1 s = 7750 tcy, 1 tcy ~ 129 s
;**********************************************************************

	list    p=12F1501			; list directive to define processor
	#include <p12F1501.inc>		; processor specific variable definitions
  	errorlevel  -302			; suppress message 302 from list file

;	__CONFIG	_CONFIG1, _FOSC_INTOSC & _WDTE_OFF & _PWRTE_OFF & _MCLRE_OFF & _CP_OFF & _BOREN_OFF & _CLKOUTEN_OFF
	__CONFIG	_CONFIG1, _FOSC_INTOSC & _WDTE_ON & _PWRTE_OFF & _MCLRE_OFF & _CP_OFF & _BOREN_OFF & _CLKOUTEN_OFF
	__CONFIG	_CONFIG2, _WRT_OFF & _STVREN_OFF & _BORV_HI & _LPBOR_OFF & _LVP_OFF

vmaxl		equ	0x40	; charge stop voltage (for example 4.2 V)
vmaxh		equ	0x41	; charge stop voltage
vwarl		equ	0x42	; warning voltage (for example 3.75 V)
vwarh		equ	0x43	; warning voltage
vlowl		equ	0x44	; low voltage (for example 3.6 V)
vlowh		equ	0x45	; low voltage
vstpl		equ	0x46	; stop voltage (for example 3.5 V)
vstph		equ	0x47	; stop voltage
;
;    voltage: 0--------Vstp-------Vlow-------Vwar-------Vmax--------
;  green LED: |---------|----------|HHHHHHHHHH|HHHHHHHHHH|HHHHHHHHHH
;    red LED: |---------|HHHHHHHHHH|HHHHHHHHHH|----------|----------
; charge out: |---------|----------|----------|----------|HHHHHHHHHH
;     system: |going OFF|......ON/OFF switch by button..............
;
fadl		equ	0x70	;vyfiltrovan hodnota men 2.stupe
fadh		equ	0x71	;vyfiltrovan hodnota men 2.stupe
avel		equ	0x72	;prmr 16 poslednch men - 1.stupe filtrace
aveh		equ	0x73	;prmr 16 poslednch men - 1.stupe filtrace
;tmp2		equ	0x72	;zpoovac smyky
;randh		equ	0x74	;genertor pseudonhodnho sla
;randl		equ	0x75	;genertor pseudonhodnho sla
wport		equ 0x75	;data na port
cycles		equ 0x76	;cykly
tmp 		equ	0x77	;cykly
func		equ	0x78	;registr mdu svcen
w_temp		equ	0x7c	; variable used for context saving 
status_temp	equ	0x7d	; variable used for context saving
poml		equ	0x7e	;pomocn pro test
pomh		equ	0x7f	;pomocn pro test

;numbf		equ	0x04	;poet funkc

#define	LEDR	LATA,0		; vstup na ervenou indikan LED
#define	LEDG	LATA,1		; vstup na zelenou indikan LED
#define	ONSW	LATA,5		; vstup pro zen P-MOSFET (0 = ON, 1 = OFF)
#define	CHRG	LATA,4		; vstup pro zen nabjen, nabj v 0, nenabj v 1
#define	SWITCH	PORTA,2		; ON/OFF tlatko
#define	SETSW	PORTA,3		; SET tlatko




	ORG 0x00
	goto start

	ORG	0x004	; interrupt vector location
; Context saving for ISR
	movwf   w_temp		; save off current W register contents
	movf	STATUS,w	; move status register into W register
	movwf	status_temp	; save off contents of STATUS register
;
	btfss	INTCON,IOCIF	; interrupt from pin level change?
	goto	out			; no
	banksel	IOCAP		;(7)
	btfss	IOCAF,2		;(7) interrupt from RA2?
	goto	clearflag	; no

; odstrann zkmit tlatka
	banksel	PORTA
	call	w50f31		;50 ms pi 31 kHz
	btfsc	SWITCH
	goto	clearflag	; tlatko u nen
	incf	func,f		; zmna stavu na nejnim bitu

clearflag
	banksel	IOCAP	;(7)
	clrf	IOCAF	;(7)
	banksel	PORTA

out		;nvrat s obnovou registr
	movf	status_temp,w	; retrieve copy of STATUS register
	movwf	STATUS			; restore pre-isr STATUS register contents
	swapf	w_temp,f
	swapf	w_temp,w		; restore pre-isr W register contents
	retfie					; return from interrupt

;-------------------------------------------------

start
; oscillator
	banksel	OSCCON	;(1)
	movlw	0x02	;LF 31 kHz
	movwf	OSCCON

; pin configuration
	banksel	LATA	;(2)
	movlw	0x30	; RA4, RA5 set to "1"
	movwf	LATA
	clrf	APFCON	;(2)

	banksel	TRISA	;(1)
	movlw	0x0c	; RA0,RA1,RA4,RA5 output, RA2-RA3 input
	movwf	TRISA
	bcf 	OPTION_REG,7;(1) enable WPU
; WDT configuration
;	banksel	WDTCON	;(1)stejn banka jako TRISA
	movlw	0x0e	; 128 ms
	movwf	WDTCON

	banksel	ANSELA	;(3)
	movlw	0x00	; digital IO
	movwf	ANSELA
	movlw	0x03	; Low-Power Sleep mode
	movwf	VREGCON

	banksel	WPUA	;(4)
	movlw	0x0c	; WPU on RA2, RA3
	movwf	WPUA

; interrupt (switch - fall edge)
	banksel	IOCAP	;(7)
	clrf	IOCAP
	movlw	0x04	; negative edge on RA2
	movwf	IOCAN
;	clrf	IOCAF	; delete possible flag
	movlw	b'10001000'	; enable interrupts, enable IOC
	movwf	INTCON	;(core)

; initialization
	banksel	PORTA	;(0)
	clrf	func	; OFF state after reset
;	incf	func,f	; ON state after reset
	clrf	FSR1H
	movlw	0x20	; prvn byte pole dat
	movwf	FSR1L

	btfss	SETSW		; je stisknuto tlatko SET?
	goto	setting		; ano - jdi na nastaven
	call	onst	; nastav, AD a PWM

loop
	call	adconvert	; vrac v fadh:fadl prmr ze 16 poslednch men
	call	volttest	; sets outputs by supply voltage
	goto	loop

setting		; nastaven napovch rovn
;ADC configuration
	banksel	ADCON0		;(1)
	movlw	b'01111101'	; ADC source = FVR, ADC is ON
	movwf	ADCON0		;(1)
	movlw	b'11110000'	; right justified, FRC oscillator, Vrpos = Vdd
	movwf	ADCON1		;(1)
	clrf	ADCON2		;(1) 0000 = No auto-conversion trigger selected
; Max. doba ustlen: kompartor 10 s, DAC 10 s, FRV 40 s, ADC ~ 5 s.
; Pi taktu 31 kHz 1 tcy ~ 129 s nen teba na ustlen ekat.

	clrf	INTCON		; zkaz peruen
; nastaven rovn vypnut Vstp
	movlw	b'00100001'	; ONSW=1 (OFF), CHRG=0, LEDG=0, LEDR=1
	call	measure		; svt erven LED
	movf	fadl,w
	movwf	vstpl
	movf	fadh,w
	movwf	vstph
; nastaven rovn Low - od tto rovn dol blik erven LED
	movlw	b'00100011'	; ONSW=1 (OFF), CHRG=0, LEDG=1, LEDR=1
	call	measure		; svt erven a zelen LED
	movf	fadl,w
	movwf	vlowl
	movf	fadh,w
	movwf	vlowh
; nastaven rovn Warning - od tto rovn dol svt erven a zelen LED
	movlw	b'00100010'	; ONSW=1 (OFF), CHRG=0, LEDG=1, LEDR=0
	call	measure		; svt zelen LED
	movf	fadl,w
	movwf	vwarl
	movf	fadh,w
	movwf	vwarh
; nastaven rovn Charge Stop - od tto rovn dol svt zelen LED
; od tto rovn nahoru je na vstup CHRG rove H
	movlw	b'00110011'	; ONSW=1 (OFF), CHRG=1, LEDG=1, LEDR=1
	call	measure		; svt erven a zelen LED, vstupy CHRG a ONSW v rovni H
	banksel PORTA		;(0)
	movf	avel,w
	movwf	vmaxl
	movf	aveh,w
	movwf	vmaxh
	movlw	b'00010000'	; ONSW=0 (ON), CHRG=1, LEDG=0, LEDR=0
	banksel	LATA		;(2)
	movwf	LATA		;(2) zhasni vechny LED + ONSW=0 (ON)
	banksel PORTA		;(0)

; uloen dat
	clrwdt
	btfsc	SETSW		; je stisknuto tlatko SET?
	goto	$-2 		; ne - ekej
	call	w50f31		;50 ms pi 31 kHz
	btfsc	SETSW		; je opravdu stisknuto tlatko SET?
	goto	$-5 		; ne - ekej
	clrwdt
	btfss	SETSW		; je u putno tlatko SET?
	goto	$-2 		; ne - ekej
	call	w50f31		;50 ms pi 31 kHz
	btfss	SETSW		; je u opravdu putno tlatko SET?
	goto	$-5 		; ne - ekej
	call	hefwrite	; ulo data do EEPROM

	banksel	LATA	;(2)
neverend
	clrwdt
	movlw	0x20		;ONSW=1 CHRG=0 LEDR=0 LEDG=0
	movwf	LATA
	call	w50f31		;50 ms pi 31 kHz
	call	w50f31		;50 ms pi 31 kHz
	movlw	0x23		;ONSW=1 CHRG=0 LEDR=1 LEDG=1
	movwf	LATA
	call	w50f31		;50 ms pi 31 kHz
	call	w50f31		;50 ms pi 31 kHz
	goto	neverend
;	reset

measure
	banksel	LATA		;(2)
	movwf	LATA		;(2) rossvi LED
	movlw	.15			; poet prvk pole
	movwf	tmp 		; new (banksel PORTA nen poteba)
	incf	tmp,f
vset
	call	adconvert	; vrac v fadh:fadl prmr ze 16 poslednch men (nvrat bank 0 PORTA)(205 tcy, 26,5 ms)
	decfsz	tmp,f		; probhlo 16 men?
	goto	vset		; ne - nov men

	btfsc	SWITCH		; je stisknuto tlatko ON/OFF?
	goto	vset-1		; ne - nov men
	clrwdt
	btfss	SWITCH		; je u putno tlatko ON/OFF?
	goto	$-2 		; ne - ekej
	call	w50f31		;50 ms pi 31 kHz
	btfss	SWITCH		; je u opravdu putno tlatko ON/OFF?
	goto	$-5 		; ne - ekej
	return

;------------ podprogramy a funkce --------------

onst	; go to ON - zapnout
	call	hefread		; ulo tabulku napt z FLASH do RAM (159 tcy i s call a return)
;ADC configuration
	banksel	ADCON0		;(1)
	movlw	b'01111100'	; ADC source = FVR, ADC is OFF
	movwf	ADCON0		;(1)
	movlw	b'11110000'	; right justified, FRC oscillator, Vrpos = Vdd
	movwf	ADCON1		;(1)
	clrf	ADCON2		;(1) 0000 = No auto-conversion trigger selected
; Max. doba ustlen: kompartor 10 s, DAC 10 s, FRV 40 s, ADC ~ 5 s.
; Pi taktu 31 kHz 1 tcy ~ 129 s nen teba na ustlen ekat.
;zapni FVR
	banksel	LATA		;(2)
	movlw	b'10000010'	; voltage reference 2.048 V enable for ADC
	movwf	FVRCON		;(2)
;spus ADC
	banksel	ADCON0		;(1)
	bsf 	ADCON0,ADON	;zapni AD pevodnk
	bsf 	ADCON0,GO	;start AD pevodu - trv 48 Tcy
	btfsc	ADCON0,GO	;done/hotovo?
	goto	$-1
;vypni ADC, DAC a FVR
	bcf 	ADCON0,ADON	;vypni AD pevodnk
	banksel	LATA		;(2)
	clrf	DACCON0		;(2)
	clrf	FVRCON		;(2)
	banksel	ADCON0		;(1)
; vpl pole dat (161 tcy)
	movlw	.15		; poet prvk pole
	movwf	cycles	; new
fillloop
	movf	ADRESL,w
	movwi	FSR1++
	movf	ADRESH,w
	movwi	FSR1++
	movlw	0x3f
	andwf	FSR1L,f
	bsf 	FSR1L,5	; rozsah 20h a 3fh
	decfsz	cycles,f		; konec?
	goto	fillloop		; ne
;pednastaven PWM - blikn
	banksel	PWM2CON		;(12)
	clrf	PWM2DCL		;(12)
	movlw	0x80		; duty cycle 50%
	movwf	PWM2DCH		;(12)
	banksel	PORTA
	movlw	0xff		; nejdel perioda
	movwf	PR2
	return

;-----------------------------------------------------------------------------
;w10f31	;10 ms pi 31 kHz 
;	movlw	0x18
;	movwf	tmp
;	decfsz	tmp,f		;
;	goto	$-1			;
;	return

;-----------------------------------------------------------------------------
w50f31	;50 ms pi 31 kHz (386 tcy, 49,8 ms)
	movlw	0x7f
	movwf	tmp
	decfsz	tmp,f		;
	goto	$-1			;
	return

;-----------------------------------------------------------------------------
adconvert	; vrac v fadh:fadl prmr ze 16 poslednch men
	clrwdt
;zapni FVR
	banksel	LATA		;(2)
	movlw	b'10000010'	; voltage reference 2.048 V enable for ADC
	movwf	FVRCON		;(2)
;spus ADC
	banksel	ADCON0		;(1)
	bsf 	ADCON0,ADON	;zapni AD pevodnk
	bsf 	ADCON0,GO	;start AD pevodu - trv 48 Tcy
	btfsc	ADCON0,GO	;done/hotovo?
	goto	$-1
;vypni ADC, DAC a FVR
	bcf 	ADCON0,ADON	;vypni AD pevodnk
	banksel	LATA		;(2)
	clrf	DACCON0		;(2)
	clrf	FVRCON		;(2)
	banksel	ADCON0		;(1)
; zpis do pole dat (7 tcy)
	movf	ADRESL,w
	movwi	FSR1++
	movf	ADRESH,w
	movwi	FSR1++
	movlw	0x3f
	andwf	FSR1L,f
	bsf 	FSR1L,5	; rozsah 20h a 3fh
; souet pole dat (163 tcy)
	banksel	PORTA
	clrf	avel
	clrf	aveh
	movlw	.16		; poet prvk pole
	movwf	cycles	; new
addloop
	moviw	FSR1++
	addwf	avel,f
	moviw	FSR1++
	addwfc	aveh,f
	movlw	0x3f
	andwf	FSR1L,f
	bsf 	FSR1L,5	; rozsah 20h a 3fh
	decfsz	cycles,f		; konec?
	goto	addloop		; ne
; dlen 16 (8 tcy)
	swapf	avel,f
	swapf	aveh,f
	movlw	0x0f
	andwf	avel,f
	movf	aveh,w
	andlw	0xf0
	iorwf	avel,f
	xorwf	aveh,f

; 2. stupe digitln filtrace - zavd 1 bit hysterezi
;N nov hodnota - aveh:avel
;P star hodnota a vsledek - fadh:fadl
	movf	aveh,w		;Nhi
	subwf	fadh,w		;tmfh,w		;Phi, subtract P-N
	btfss	STATUS,Z	;jsou stejn? - Are they equal?
	goto	fi2fhi		;ne
	movf	avel,w		;Nlo, ano, horn bajty jsou stejn (yes, they are equal - compare lo)
	subwf	fadl,w		;tmfl,w		;Plo, subtract P-X
fi2fhi
; if N=P then now Z=1.
; if N>P then now C=0.
; if N<=P then now C=1.

; Filtrace (N - nov hodnota, P - hodnota pro displej
;    N <= P -> P = N		(Z=0, C=1 - zkopruj)
;    N > P  -> P = N-1		(Z=0, C=0 - odeti 1 a zkopruj)
	btfsc	STATUS,C	;
	goto	fi2copy		;jen zkoprovn
; -1 a zkoprovn
	movlw	.1			;
	subwf	avel,w		;
	movwf	fadl		;
	movf	aveh,w		;
	btfss	STATUS,C	;
	decf	aveh,w		;
	movwf	fadh		;
	return
fi2copy	 ;zkoprovn
	movf	aveh,w		;
	movwf	fadh		;
	movf	avel,w
	movwf	fadl		;
	return

;-----------------------------------------------------------------------------
volttest	; sets outputs by supply voltage
		; vmaxl		equ	0x40	; charge stop voltage (for example 4.2 V)
		; vmaxh		equ	0x41	; charge stop voltage
		; vwarl		equ	0x42	; warning voltage (for example 3.75 V)
		; vwarh		equ	0x43	; warning voltage
		; vlowl		equ	0x44	; low voltage (for example 3.6 V)
		; vlowh		equ	0x45	; low voltage
		; vstpl		equ	0x46	; stop voltage (for example 3.5 V)
		; vstph		equ	0x47	; stop voltage

; output stop voltage (compare_unsigned_16)	; Vdd <= Vlow
	movf	fadh,w		;Nhi
	subwf	vstph,w		;Phi, subtract P-N
	btfss	STATUS,Z	;jsou stejn? - Are they equal?
	goto	resulthi1	;ne
	movf	fadl,w		;Nlo, ano, horn bajty jsou stejn (yes, they are equal - compare lo)
	subwf	vstpl,w		;Plo, subtract P-X
resulthi1
; if N=P then now Z=1.
; if N>P then now C=0.
; if N<=P then now C=1.
;    N <= P -> P = N		(Z=0, C=1 - dal test)
;    N > P  -> P = N-1		(Z=0, C=0 - nastav vstupy a nvrat)
	btfsc	STATUS,C	;
	goto	tstlowvolt	; Vdd > Vstp
	clrf	func		; system go OFF - low voltage
	banksel	PWM2CON		;(12) zastav PWM (blikn)
	clrf	PWM2CON

	banksel	PORTA		;(0)
	clrf	T2CON		;disable TIMER2
	movlw	0x20		;ONSW=1 CHRG=0 LEDR=0 LEDG=0
;	movwf	wport		;(0)
	goto	portwrite	;  ne

tstlowvolt	; Vdd > Vstp
	btfss	func,0		; je vypnuto (func,0 = 0)?
	goto	chargeoff	;  ano, testuj jen nabjen

	movf	fadh,w		;Nhi
	subwf	vlowh,w		;Phi, subtract P-N
	btfss	STATUS,Z	;jsou stejn? - Are they equal?
	goto	resulthi2	;ne
	movf	fadl,w		;Nlo, ano, horn bajty jsou stejn (yes, they are equal - compare lo)
	subwf	vlowl,w		;Plo, subtract P-X
resulthi2
	btfsc	STATUS,C	;
	goto	tstwarning	; Vdd > Vlow
;nastaven PWM - blikn erven LED
	bcf 	PIR1,TMR2IF	;clear flag (podle nvodu)
;	movlw	0x05
	movlw	0x06
	movwf	T2CON		;enable TIMER2, prescaler = 4 (5) nebo 16 (6)
	banksel	PWM2CON		;(12)
	movlw	0xd0		; PWM enable, output enable, active low
	movwf	PWM2CON		;(12)
;	banksel	PORTA		;(0)
	movlw	b'00000001'	; ONSW=0 (ON), CHRG=0 (yes), LEDG=0, LEDR=1
	goto	portwrite

tstwarning	; Vdd > Vlow
;zastav PWM
	banksel	PWM2CON		;(12) zastav PWM
	clrf	PWM2CON
;	banksel	TRISA		;(1)
;	bcf 	TRISA,0		; pipoj RA0 k LATA
	banksel	PORTA		;(0)
	clrf	T2CON		;disable TIMER2

	movf	fadh,w		;Nhi
	subwf	vwarh,w		;Phi, subtract P-N
	btfss	STATUS,Z	;jsou stejn? - Are they equal?
	goto	resulthi3	;ne
	movf	fadl,w		;Nlo, ano, horn bajty jsou stejn (yes, they are equal - compare lo)
	subwf	vwarl,w		;Plo, subtract P-X
resulthi3
	btfsc	STATUS,C	;
	goto	tstchrgvolt	; Vdd > Vwar
	movlw	b'00000011'	; ONSW=0 (ON), CHRG=0 (yes), LEDG=1, LEDR=1
;	movwf	wport		;(0)
	goto	portwrite

tstchrgvolt	; Vdd > Vwar
	movf	fadh,w		;Nhi
	subwf	vmaxh,w		;Phi, subtract P-N
	btfss	STATUS,Z	;jsou stejn? - Are they equal?
	goto	resulthi4	;ne
	movf	fadl,w		;Nlo, ano, horn bajty jsou stejn (yes, they are equal - compare lo)
	subwf	vmaxl,w		;Plo, subtract P-X
resulthi4
	btfsc	STATUS,C	;
	goto	chargestop	; Vdd > Vmax
	movlw	b'00000010'	; ONSW=0 (ON), CHRG=0 (yes), LEDG=1, LEDR=0
;	movwf	wport		;(0)
	goto	portwrite
chargestop	; Vdd > Vmax
	movlw	b'00010010'	; ONSW=0 (ON), CHRG=1 (stop), LEDG=1, LEDR=0
;	movwf	wport		;(0)
	goto	portwrite

chargeoff	; Vdd > Vwar
	banksel	PWM2CON		;(12) zastav PWM
	clrf	PWM2CON
	banksel	PORTA		;(0)
	movf	fadh,w		;Nhi
	subwf	vmaxh,w		;Phi, subtract P-N
	btfss	STATUS,Z	;jsou stejn? - Are they equal?
	goto	resulthi5	;ne
	movf	fadl,w		;Nlo, ano, horn bajty jsou stejn (yes, they are equal - compare lo)
	subwf	vmaxl,w		;Plo, subtract P-X
resulthi5
	btfsc	STATUS,C	;
	goto	chrgstopoff	; Vdd > Vmax
	movlw	b'00100000'	; ONSW=1 (OFF), CHRG=0 (yes), LEDG=0, LEDR=0
;	movwf	wport		;(0)
	goto	portwrite
chrgstopoff	; Vdd > Vmax
	movlw	b'00110000'	; ONSW=1 (OFF), CHRG=1 (stop), LEDG=0, LEDR=0
;	movwf	wport		;(0)

portwrite	
;	movf	wport,w
	banksel	LATA		;(2)
	movwf	LATA		;(2)
	banksel PORTA		;(0)
	return

;-----------------------------------------------------------------------------
hefread	;pete 16 byte z HEF od adresy 0x03d0 (ni byte pamovho msta)
		; a ulo je do RAM na adresy 0x40 a 0x4f
	clrf	FSR0H
	movlw	0x40
	movwf	FSR0L	; adresa 40h
	movlw   .16
	movwf	cycles
	banksel	PMADRL
	movlw	0x03
	movwf	PMADRH	; adresa H 03h
	movlw	0xd0
	movwf	PMADRL	; adresa L 80h
	bcf 	PMCON1,CFGS
hr_next
	bsf 	PMCON1,RD
	nop
	nop
	movf	PMDATL,w
	movwi	FSR0++
	incf	PMADRL,f
	decfsz  cycles,f
	goto	hr_next
	banksel PORTA
	return
;-----------------------------------------------------------------------------
hefwrite	; zape 16 byte RAM od adresy 40h do HEF od adresy 3d0h
;erase
	bcf 	INTCON,GIE		;disable interrupts
	banksel	PMADRL			;(3)
	movlw	0x03
	movwf	PMADRH			;(3) adresa H 03h
	movlw	0xd0
	movwf	PMADRL			;(3) adresa L 80h
	bcf 	PMCON1,CFGS		;(3) Not configuration space
	bsf 	PMCON1,FREE		;(3) Specify an erase operation
	bsf 	PMCON1,WREN		;(3) Enable writes
	movlw	0x55			; Start of required sequence to initiate erase
	movwf	PMCON2			;(3)
	movlw	0xaa
	movwf	PMCON2			;(3)
	bsf 	PMCON1,WR		;(3) Set WR bit to begin erase
	nop
	nop
	bcf 	PMCON1,WREN		;(3) Disable writes
	clrwdt

;write
	movlw	0x03
	movwf	PMADRH			;(3) adresa H 03h
	movlw	0xd0
	movwf	PMADRL			;(3) adresa L d0h

	clrf	FSR0H
	movlw	0x40
	movwf	FSR0L	; adresa 40h

	bcf 	PMCON1,CFGS		;(3) Not configuration space
	bsf 	PMCON1,WREN		;(3) Enable writes
	bsf 	PMCON1,LWLO		;(3) Only Load Write Latches
LOOP
	moviw	FSR0++			; Load first data byte into lower
	movwf	PMDATL			;(3)
;	moviw	FSR0++			; Load second data byte into upper
	movlw	0x3f			; j do vho bajtu nic nezapisuji
	movwf	PMDATH			;(3)
	movf	PMADRL,W		; Check if lower bits of address are '00000'
	xorlw	0x0f			; Check if we're on the last of 16 addresses
	andlw	0x0f
	btfsc	STATUS,Z		; Exit if last of 16 words,
	goto	START_WRITE

	movlw	0x55			; Start of required write sequence
	movwf	PMCON2			;(3)
	movlw	0xaa
	movwf	PMCON2			;(3)
	bsf 	PMCON1,WR		;(3) Set WR bit to begin write
	nop
	nop

	incf	PMADRL,f		;(3) Still loading latches Increment address
	goto	LOOP			; Write next latches
START_WRITE
	bcf 	PMCON1,LWLO		; No more loading latches - Actually start Flash program memory write
	movlw	0x55			; Start of required write sequence
	movwf	PMCON2
	movlw	0xaa
	movwf	PMCON2
	bsf 	PMCON1,WR		; Set WR bit to begin write
	nop
	nop
	bcf 	PMCON1,WREN		; Disable writes
;	bsf 	INTCON,GIE		; Enable interrupts (if needed)
	banksel PORTA
	return
;-----------------------------------------------------------------------------


	ORG 0x3d0
	DW	0x3fff, 0x3f01	;Vmax 01ffh ~ 4.2 V
	DW	0x3f2f, 0x3f02	;Vwar 022fh ~ 3.75 V
	DW	0x3f46, 0x3f02	;Vlow 0246h ~ 3.6 V
	DW	0x3f57, 0x3f02	;Vstp 0257h ~ 3.5 V
;	DW	0x3f33, 0x3f77	;Vblk	rezerva

	ORG 0x3e0
	DE	"spinac-nabijeckav1.0     VH 2017"

	END